@@ -13,7 +13,8 @@
 """
 List resources from the Network service.
 
-For a full guide see TODO(etoews):link to docs on developer.openstack.org
+For a full guide see
+https://docs.openstack.org/openstacksdk/latest/user/guides/network.html
 """
 
 
@@ -50,3 +51,10 @@ def list_routers(conn):
 
     for router in conn.network.routers():
         print(router)
+
+
+def list_network_agents(conn):
+    print("List Network Agents:")
+
+    for agent in conn.network.agents():
+        print(agent)
