@@ -18,6 +18,7 @@ import warnings
 from openstack import exceptions
 from openstack.image.v2 import cache as _cache
 from openstack.image.v2 import image as _image
+from openstack.image.v2 import image_tasks as _image_tasks
 from openstack.image.v2 import member as _member
 from openstack.image.v2 import metadef_namespace as _metadef_namespace
 from openstack.image.v2 import metadef_object as _metadef_object
@@ -90,8 +91,8 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image`
             instance.
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.NotFoundException` will be raised when
-            the metadef namespace does not exist.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the metadef namespace does not exist.
         :returns: ``None``
         """
         return self._delete(_cache.Cache, image, ignore_missing=ignore_missing)
@@ -131,6 +132,11 @@ class Proxy(proxy.Proxy):
         timeout=3600,
         validate_checksum=False,
         use_import=False,
+        import_method=None,
+        uri=None,
+        remote_region=None,
+        remote_image_id=None,
+        remote_service_interface=None,
         stores=None,
         all_stores=None,
         all_stores_must_succeed=None,
@@ -198,6 +204,20 @@ class Proxy(proxy.Proxy):
             cloud to transform image format. If the cloud has disabled direct
             uploads, this will default to true. If you wish to use other import
             methods, use the ``import_image`` method instead.
+        :param import_method: Method to use for importing the image. Not all
+            deployments support all methods. One of: ``glance-direct``
+            (default), ``web-download``, ``glance-download`` (``copy-image`` is
+            not used with create). Use of ``glance-direct`` requires the image
+            be first staged.
+        :param uri: Required only if using the ``web-download`` import method.
+            This url is where the data is made available to the Image
+            service.
+        :param remote_region: The remote glance region to download the image
+            from when using glance-download.
+        :param remote_image_id: The ID of the image to import from the
+            remote glance when using glance-download.
+        :param remote_service_interface: The remote glance service interface to
+            use when using glance-download.
         :param stores: List of stores to be used when enabled_backends is
             activated in glance. List values can be the id of a store or a
             :class:`~openstack.image.v2.service_info.Store` instance.
@@ -322,7 +342,7 @@ class Proxy(proxy.Proxy):
         if tags:
             image_kwargs['tags'] = tags
 
-        if filename or data:
+        if filename or data or import_method:
             image = self._upload_image(
                 name,
                 filename=filename,
@@ -332,6 +352,11 @@ class Proxy(proxy.Proxy):
                 timeout=timeout,
                 validate_checksum=validate_checksum,
                 use_import=use_import,
+                import_method=import_method,
+                uri=uri,
+                remote_region=remote_region,
+                remote_image_id=remote_image_id,
+                remote_service_interface=remote_service_interface,
                 stores=stores,
                 all_stores=all_stores,
                 all_stores_must_succeed=all_stores_must_succeed,
@@ -547,6 +572,11 @@ class Proxy(proxy.Proxy):
         timeout=None,
         validate_checksum=True,
         use_import=False,
+        import_method=None,
+        uri=None,
+        remote_region=None,
+        remote_image_id=None,
+        remote_service_interface=None,
         stores=None,
         all_stores=None,
         all_stores_must_succeed=None,
@@ -589,6 +619,11 @@ class Proxy(proxy.Proxy):
                     meta=meta,
                     validate_checksum=validate_checksum,
                     use_import=use_import,
+                    import_method=import_method,
+                    uri=uri,
+                    remote_region=remote_region,
+                    remote_image_id=remote_image_id,
+                    remote_service_interface=remote_service_interface,
                     stores=stores,
                     all_stores=all_stores,
                     all_stores_must_succeed=all_stores_must_succeed,
@@ -598,7 +633,9 @@ class Proxy(proxy.Proxy):
             self.log.debug("Image creation failed", exc_info=True)
             raise
         except Exception as e:
-            raise exceptions.SDKException(f"Image creation failed: {str(e)}")
+            raise exceptions.SDKException(
+                f"Image creation failed: {e!s}"
+            ) from e
 
     def _make_v2_image_params(self, meta, properties):
         ret: dict = {}
@@ -623,15 +660,28 @@ class Proxy(proxy.Proxy):
         meta,
         validate_checksum,
         use_import=False,
+        import_method=None,
+        uri=None,
+        remote_region=None,
+        remote_image_id=None,
+        remote_service_interface=None,
         stores=None,
         all_stores=None,
         all_stores_must_succeed=None,
         **image_kwargs,
     ):
+        if all_stores and stores:
+            raise exceptions.InvalidRequest(
+                "all_stores is mutually exclusive with stores"
+            )
+
         # use of any of these imply use_import=True
         if stores or all_stores or all_stores_must_succeed:
             use_import = True
 
+        if use_import and not import_method:
+            import_method = 'glance-direct'
+
         if filename and not data:
             image_data = open(filename, 'rb')
         else:
@@ -647,7 +697,7 @@ class Proxy(proxy.Proxy):
 
         supports_import = (
             image.image_import_methods
-            and 'glance-direct' in image.image_import_methods
+            and import_method in image.image_import_methods
         )
         if use_import and not supports_import:
             raise exceptions.SDKException(
@@ -660,8 +710,23 @@ class Proxy(proxy.Proxy):
                 response = image.upload(self)
                 exceptions.raise_from_response(response)
             if use_import:
-                image.stage(self)
-                image.import_image(self)
+                kwargs = {}
+                if stores is not None:
+                    kwargs['stores'] = stores
+                else:
+                    kwargs['all_stores'] = all_stores
+                    kwargs['all_stores_must_succeed'] = all_stores_must_succeed
+                if import_method == 'glance-direct':
+                    image.stage(self)
+                elif import_method == 'web-download':
+                    kwargs['uri'] = uri
+                elif import_method == 'glance-download':
+                    kwargs.update(
+                        remote_region=remote_region,
+                        remote_image_id=remote_image_id,
+                        remote_service_interface=remote_service_interface,
+                    )
+                self.import_image(image, method=import_method, **kwargs)
 
             # image_kwargs are flat here
             md5 = image_kwargs.get(self._IMAGE_MD5_KEY)
@@ -694,9 +759,9 @@ class Proxy(proxy.Proxy):
     ):
         if not self._connection.has_service('object-store'):
             raise exceptions.SDKException(
-                f"The cloud {self._connection.config.name} is configured to use tasks for image "
-                "upload, but no object-store service is available. "
-                "Aborting."
+                f"The cloud {self._connection.config.name} is configured to "
+                f"use tasks for image upload, but no object-store service is "
+                f"available. Aborting."
             )
 
         properties = image_kwargs.get('properties', {})
@@ -761,7 +826,7 @@ class Proxy(proxy.Proxy):
                 raise exceptions.SDKException(
                     f"Image creation failed: {e.message}",
                     extra_data=glance_task,
-                )
+                ) from e
             finally:
                 # Clean up after ourselves. The object we created is not
                 # needed after the import is done.
@@ -960,13 +1025,25 @@ class Proxy(proxy.Proxy):
 
         return True
 
+    def image_tasks(self, image):
+        """Return a generator of Image Tasks
+
+        :param image: The value can be either the name of an image or a
+            :class:`~openstack.image.v2.image.Image` instance.
+        :return: A generator object of image tasks
+        :rtype: :class: ~openstack.image.v2.image_tasks.ImageTasks
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+                when no resource can be found.
+        """
+        image_id = resource.Resource._get_id(image)
+        return self._list(_image_tasks.ImageTasks, image_id=image_id)
+
     def add_tag(self, image, tag):
         """Add a tag to an image
 
         :param image: The value can be the ID of a image or a
-            :class:`~openstack.image.v2.image.Image` instance
-            that the member will be created for.
-        :param str tag: The tag to be added
+            :class:`~openstack.image.v2.image.Image` instance.
+        :param tag: The tag to be added.
 
         :returns: None
         """
@@ -974,12 +1051,11 @@ class Proxy(proxy.Proxy):
         image.add_tag(self, tag)
 
     def remove_tag(self, image, tag):
-        """Remove a tag to an image
+        """Remove a tag from an image
 
         :param image: The value can be the ID of a image or a
-            :class:`~openstack.image.v2.image.Image` instance
-            that the member will be created for.
-        :param str tag: The tag to be removed
+            :class:`~openstack.image.v2.image.Image` instance.
+        :param tag: The tag to be removed.
 
         :returns: None
         """
@@ -1016,9 +1092,9 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image` instance that the member
             is part of. This is required if ``member`` is an ID.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.NotFoundException` will be raised when
-            the member does not exist. When set to ``True``, no exception will
-            be set when attempting to delete a nonexistent member.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the member does not exist. When set to ``True``, no exception
+            will be set when attempting to delete a nonexistent member.
 
         :returns: ``None``
         """
@@ -1109,6 +1185,7 @@ class Proxy(proxy.Proxy):
         image_id = resource.Resource._get_id(image)
         return self._update(
             _member.Member,
+            None,
             member_id=member_id,
             image_id=image_id,
             **attrs,
@@ -1135,8 +1212,8 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance.
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.NotFoundException` will be raised when
-            the metadef namespace does not exist.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the metadef namespace does not exist.
         :returns: ``None``
         """
         self._delete(
@@ -1199,6 +1276,50 @@ class Proxy(proxy.Proxy):
             **attrs,
         )
 
+    def add_tag_to_metadef_namespace(self, namespace, tag):
+        """Add a tag to a metadef namespace
+
+        :param metadef_namespace: Either the name of a metadef namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param str tag: The tag to be added.
+
+        :returns: None
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, namespace
+        )
+        namespace.add_tag(self, tag)
+
+    def remove_tag_from_metadef_namespace(self, namespace, tag):
+        """Remove a tag from a metadef namespace
+
+        :param metadef_namespace: Either the name of a metadef namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param str tag: The tag to be removed.
+
+        :returns: None
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, namespace
+        )
+        namespace.remove_tag(self, tag)
+
+    def remove_tags_from_metadef_namespace(self, namespace):
+        """Remove all tags from a metadef namespace
+
+        :param metadef_namespace: Either the name of a metadef namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+
+        :returns: None
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, namespace
+        )
+        namespace.remove_all_tags(self)
+
     # ====== METADEF OBJECT ======
     def create_metadef_object(self, namespace, **attrs):
         """Create a new object from namespace
@@ -1381,8 +1502,8 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.NotFoundException` will be raised when
-            the metadef resource type association does not exist.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the metadef resource type association does not exist.
         :returns: ``None``
         """
         namespace_name = resource.Resource._get_id(metadef_namespace)
@@ -1540,7 +1661,7 @@ class Proxy(proxy.Proxy):
         )
 
     def delete_all_metadef_properties(self, metadef_namespace):
-        """Delete all metadata definitions property inside a specific namespace.
+        """Delete all metadata definitions property inside a namespace.
 
         :param metadef_namespace: The value can be either the name of a metadef
             namespace or a
@@ -1899,11 +2020,11 @@ class Proxy(proxy.Proxy):
         self,
         res: resource.ResourceT,
         status: str,
-        failures: ty.Optional[list[str]] = None,
-        interval: ty.Union[int, float, None] = 2,
-        wait: ty.Optional[int] = None,
+        failures: list[str] | None = None,
+        interval: int | float | None = 2,
+        wait: int | None = None,
         attribute: str = 'status',
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
+        callback: ty.Callable[[int], None] | None = None,
     ) -> resource.ResourceT:
         """Wait for the resource to be in a particular status.
 
@@ -1939,7 +2060,7 @@ class Proxy(proxy.Proxy):
         res: resource.ResourceT,
         interval: int = 2,
         wait: int = 120,
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
+        callback: ty.Callable[[int], None] | None = None,
     ) -> resource.ResourceT:
         """Wait for a resource to be deleted.
 
