@@ -12,11 +12,13 @@
 
 from calendar import timegm
 import collections
+import functools
 from hashlib import sha1
 import hmac
 import json
 import os
 import time
+import typing as ty
 from urllib import parse
 
 from openstack import _log
@@ -27,6 +29,7 @@ from openstack.object_store.v1 import container as _container
 from openstack.object_store.v1 import info as _info
 from openstack.object_store.v1 import obj as _obj
 from openstack import proxy
+from openstack import resource
 from openstack import utils
 
 DEFAULT_OBJECT_SEGMENT_SIZE = 1073741824  # 1GB
@@ -55,6 +58,7 @@ class Proxy(proxy.Proxy):
 
     log = _log.setup_logging('openstack')
 
+    @functools.lru_cache(maxsize=256)
     def _extract_name(self, url, service_type=None, project_id=None):
         url_path = parse.urlparse(url).path.strip()
         # Remove / from the beginning to keep the list indexes of interesting
@@ -156,9 +160,10 @@ class Proxy(proxy.Proxy):
         :param container: The value can be either the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
-            the container does not exist. When set to ``True``, no exception
-            will be set when attempting to delete a nonexistent server.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the container does not exist. When set to ``True``, no
+            exception will be set when attempting to delete a nonexistent
+            server.
 
         :returns: ``None``
         """
@@ -173,7 +178,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.object_store.v1.container.Container` instance.
 
         :returns: One :class:`~openstack.object_store.v1.container.Container`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._head(_container.Container, container)
@@ -278,7 +283,7 @@ class Proxy(proxy.Proxy):
 
         :returns: Instance of the
             :class:`~openstack.object_store.v1.obj.Object` objects.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj=obj, container=container)
@@ -322,7 +327,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
 
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj=obj, container=container)
@@ -339,7 +344,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
 
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         :returns: An iterator that iterates over chunk_size bytes
         """
@@ -394,7 +399,8 @@ class Proxy(proxy.Proxy):
         :param metadata: This dict will get changed into headers that set
             metadata of the object
 
-        :raises: ``OpenStackCloudException`` on operation error.
+        :raises: ``:class:`~openstack.exceptions.SDKException``` on operation
+            error.
         """
         if data is not None and filename:
             raise ValueError(
@@ -426,9 +432,7 @@ class Proxy(proxy.Proxy):
             metadata[self._connection._OBJECT_SHA256_KEY] = sha256
 
         container_name = self._get_container_name(container=container)
-        endpoint = '{container}/{name}'.format(
-            container=container_name, name=name
-        )
+        endpoint = f'{container_name}/{name}'
 
         if data is not None:
             self.log.debug(
@@ -489,9 +493,9 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the ID of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
-            the object does not exist.  When set to ``True``, no exception will
-            be set when attempting to delete a nonexistent server.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the object does not exist.  When set to ``True``, no exception
+            will be set when attempting to delete a nonexistent server.
 
         :returns: ``None``
         """
@@ -513,7 +517,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.object_store.v1.container.Container` instance.
 
         :returns: One :class:`~openstack.object_store.v1.obj.Object`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj, container)
@@ -578,9 +582,7 @@ class Proxy(proxy.Proxy):
             metadata = self.get_object_metadata(name, container).metadata
         except exceptions.NotFoundException:
             self._connection.log.debug(
-                "swift stale check, no object: {container}/{name}".format(
-                    container=container, name=name
-                )
+                f"swift stale check, no object: {container}/{name}"
             )
             return True
 
@@ -604,7 +606,7 @@ class Proxy(proxy.Proxy):
         if not up_to_date:
             self._connection.log.debug(
                 "swift checksum mismatch: "
-                " %(filename)s!=%(container)s/%(name)s",
+                "%(filename)s!=%(container)s/%(name)s",
                 {'filename': filename, 'container': container, 'name': name},
             )
             return True
@@ -650,7 +652,7 @@ class Proxy(proxy.Proxy):
                     # While Object Storage usually expects the name to be
                     # urlencoded in most requests, the SLO manifest requires
                     # plain object names instead.
-                    path='/{name}'.format(name=parse.unquote(name)),
+                    path=f'/{parse.unquote(name)}',
                     size_bytes=segment.length,
                 )
             )
@@ -754,9 +756,7 @@ class Proxy(proxy.Proxy):
                 offset,
                 segment_size if segment_size < remaining else remaining,
             )
-            name = '{endpoint}/{index:0>6}'.format(
-                endpoint=endpoint, index=index
-            )
+            name = f'{endpoint}/{index:0>6}'
             segments[name] = segment
         return segments
 
@@ -768,15 +768,17 @@ class Proxy(proxy.Proxy):
         try:
             # caps = self.get_object_capabilities()
             caps = self.get_info()
-        except exceptions.SDKException as e:
-            if e.response.status_code in (404, 412):
-                server_max_file_size = DEFAULT_MAX_FILE_SIZE
-                self._connection.log.info(
-                    "Swift capabilities not supported. "
-                    "Using default max file size."
-                )
-            else:
-                raise
+        except (
+            exceptions.NotFoundException,
+            exceptions.PreconditionFailedException,
+        ):
+            server_max_file_size = DEFAULT_MAX_FILE_SIZE
+            self._connection.log.info(
+                "Swift capabilities not supported. "
+                "Using default max file size."
+            )
+        except exceptions.SDKException:
+            raise
         else:
             server_max_file_size = caps.swift.get('max_file_size', 0)
             min_segment_size = caps.slo.get('min_segment_size', 0)
@@ -804,7 +806,7 @@ class Proxy(proxy.Proxy):
                 continue
             name = self._object_name_from_url(result.url)
             for entry in manifest:
-                if entry['path'] == '/{name}'.format(name=parse.unquote(name)):
+                if entry['path'] == f'/{parse.unquote(name)}':
                     entry['etag'] = result.headers['Etag']
 
     def get_info(self):
@@ -872,8 +874,8 @@ class Proxy(proxy.Proxy):
             temp_url_key = self.get_temp_url_key(container)
         if not temp_url_key:
             raise exceptions.SDKException(
-                'temp_url_key was not given, nor was a temporary url key'
-                ' found for the account or the container.'
+                'temp_url_key was not given, nor was a temporary url key '
+                'found for the account or the container.'
             )
         return temp_url_key
 
@@ -925,17 +927,16 @@ class Proxy(proxy.Proxy):
 
         res = self._get_resource(_container.Container, container)
         endpoint = parse.urlparse(self.get_endpoint())
-        path = '/'.join([endpoint.path, res.name, object_prefix])
+        if isinstance(endpoint.path, bytes):
+            # To keep mypy happy: the output type will be the same as the input
+            # type
+            path = endpoint.path.decode()
+        else:
+            path = endpoint.path
+        path = '/'.join([path, res.name, object_prefix])
 
-        data = '%s\n%s\n%s\n%s\n%s' % (
-            path,
-            redirect_url,
-            max_file_size,
-            max_upload_count,
-            expires,
-        )
-        data = data.encode('utf8')
-        sig = hmac.new(temp_url_key, data, sha1).hexdigest()
+        data = f'{path}\n{redirect_url}\n{max_file_size}\n{max_upload_count}\n{expires}'  # noqa: E501
+        sig = hmac.new(temp_url_key, data.encode(), sha1).hexdigest()
 
         return (expires, sig)
 
@@ -991,18 +992,17 @@ class Proxy(proxy.Proxy):
                     try:
                         t = time.strptime(seconds, f)
                     except ValueError:
-                        t = None
+                        continue
+
+                    if f == EXPIRES_ISO8601_FORMAT:
+                        timestamp = timegm(t)
                     else:
-                        if f == EXPIRES_ISO8601_FORMAT:
-                            timestamp = timegm(t)
-                        else:
-                            # Use local time if UTC designator is missing.
-                            timestamp = int(time.mktime(t))
+                        # Use local time if UTC designator is missing.
+                        timestamp = int(time.mktime(t))
 
-                        absolute = True
-                        break
-
-                if t is None:
+                    absolute = True
+                    break
+                else:
                     raise ValueError()
             else:
                 if not timestamp.is_integer():
@@ -1045,6 +1045,7 @@ class Proxy(proxy.Proxy):
                 method.upper(),
             )
 
+        expiration: float | int
         if not absolute:
             expiration = _get_expiration(timestamp)
         else:
@@ -1062,9 +1063,9 @@ class Proxy(proxy.Proxy):
                     ip_range = ip_range.decode('utf-8')
                 except UnicodeDecodeError:
                     raise ValueError('ip_range must be representable as UTF-8')
-            hmac_parts.insert(0, "ip=%s" % ip_range)
+            hmac_parts.insert(0, f"ip={ip_range}")
 
-        hmac_body = u'\n'.join(hmac_parts)
+        hmac_body = '\n'.join(hmac_parts)
 
         temp_url_key = self._check_temp_url_key(temp_url_key=temp_url_key)
 
@@ -1073,19 +1074,19 @@ class Proxy(proxy.Proxy):
         ).hexdigest()
 
         if iso8601:
-            expiration = time.strftime(
+            exp = time.strftime(
                 EXPIRES_ISO8601_FORMAT, time.gmtime(expiration)
             )
+        else:
+            exp = str(expiration)
 
-        temp_url = u'{path}?temp_url_sig={sig}&temp_url_expires={exp}'.format(
-            path=path_for_body, sig=sig, exp=expiration
-        )
+        temp_url = f'{path_for_body}?temp_url_sig={sig}&temp_url_expires={exp}'
 
         if ip_range:
-            temp_url += u'&temp_url_ip_range={}'.format(ip_range)
+            temp_url += f'&temp_url_ip_range={ip_range}'
 
         if prefix:
-            temp_url += u'&temp_url_prefix={}'.format(parts[4])
+            temp_url += f'&temp_url_prefix={parts[4]}'
         # Have return type match path from caller
         if isinstance(path, bytes):
             return temp_url.encode('utf-8')
@@ -1123,6 +1124,69 @@ class Proxy(proxy.Proxy):
                 deleted = True
         return deleted
 
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: list[str] | None = None,
+        interval: int | float | None = 2,
+        wait: int | None = None,
+        attribute: str = 'status',
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
+
     # ========== Project Cleanup ==========
     def _get_cleanup_dependencies(self):
         return {'object_store': {'before': []}}
@@ -1142,17 +1206,18 @@ class Proxy(proxy.Proxy):
             return
 
         is_bulk_delete_supported = False
-        bulk_delete_max_per_request = None
+        bulk_delete_max_per_request = 1
         try:
             caps = self.get_info()
         except exceptions.SDKException:
             pass
         else:
-            bulk_delete = caps.swift.get("bulk_delete", {})
-            is_bulk_delete_supported = bulk_delete is not None
-            bulk_delete_max_per_request = bulk_delete.get(
-                "max_deletes_per_request", 100
-            )
+            bulk_delete = caps.get("bulk_delete")
+            if bulk_delete is not None:
+                is_bulk_delete_supported = True
+                bulk_delete_max_per_request = bulk_delete.get(
+                    "max_deletes_per_request", 10000
+                )
 
         elements = []
         for cont in self.containers():
