@@ -32,18 +32,23 @@ converted into this Resource class' appropriate components and types
 and then returned to the caller.
 """
 
+from __future__ import annotations
+
+import builtins
 import collections
+import collections.abc
 import inspect
 import itertools
 import operator
 import typing as ty
-import typing_extensions as ty_ext
 import urllib.parse
 import warnings
 
 import jsonpatch
 from keystoneauth1 import adapter
 from keystoneauth1 import discover
+import requests
+import typing_extensions as ty_ext
 
 from openstack import _log
 from openstack import exceptions
@@ -51,23 +56,29 @@ from openstack import fields
 from openstack import utils
 from openstack import warnings as os_warnings
 
+if ty.TYPE_CHECKING:
+    from openstack import connection
+
 LOG = _log.setup_logging(__name__)
 
+AdapterT = ty.TypeVar('AdapterT', bound=adapter.Adapter)
+ResourceT = ty.TypeVar('ResourceT', bound='Resource')
+
 
 # TODO(stephenfin): We should deprecate the 'type' and 'list_type' arguments
 # for all of the below in favour of annotations. To that end, we have stuck
 # with Any rather than generating super complex types
 def Body(
     name: str,
-    type: ty.Optional[ty.Any] = None,
+    type: ty.Any | None = None,
     default: ty.Any = None,
-    alias: ty.Optional[str] = None,
-    aka: ty.Optional[str] = None,
+    alias: str | None = None,
+    aka: str | None = None,
     alternate_id: bool = False,
-    list_type: ty.Optional[ty.Any] = None,
+    list_type: ty.Any | None = None,
     coerce_to_default: bool = False,
     deprecated: bool = False,
-    deprecation_reason: ty.Optional[str] = None,
+    deprecation_reason: str | None = None,
 ) -> ty.Any:
     return fields.Body(
         name,
@@ -85,15 +96,15 @@ def Body(
 
 def Header(
     name: str,
-    type: ty.Optional[ty.Any] = None,
+    type: ty.Any | None = None,
     default: ty.Any = None,
-    alias: ty.Optional[str] = None,
-    aka: ty.Optional[str] = None,
+    alias: str | None = None,
+    aka: str | None = None,
     alternate_id: bool = False,
-    list_type: ty.Optional[ty.Any] = None,
+    list_type: ty.Any | None = None,
     coerce_to_default: bool = False,
     deprecated: bool = False,
-    deprecation_reason: ty.Optional[str] = None,
+    deprecation_reason: str | None = None,
 ) -> ty.Any:
     return fields.Header(
         name,
@@ -111,15 +122,15 @@ def Header(
 
 def URI(
     name: str,
-    type: ty.Optional[ty.Any] = None,
+    type: ty.Any | None = None,
     default: ty.Any = None,
-    alias: ty.Optional[str] = None,
-    aka: ty.Optional[str] = None,
+    alias: str | None = None,
+    aka: str | None = None,
     alternate_id: bool = False,
-    list_type: ty.Optional[ty.Any] = None,
+    list_type: ty.Any | None = None,
     coerce_to_default: bool = False,
     deprecated: bool = False,
-    deprecation_reason: ty.Optional[str] = None,
+    deprecation_reason: str | None = None,
 ) -> ty.Any:
     return fields.URI(
         name,
@@ -137,15 +148,15 @@ def URI(
 
 def Computed(
     name: str,
-    type: ty.Optional[ty.Any] = None,
+    type: ty.Any | None = None,
     default: ty.Any = None,
-    alias: ty.Optional[str] = None,
-    aka: ty.Optional[str] = None,
+    alias: str | None = None,
+    aka: str | None = None,
     alternate_id: bool = False,
-    list_type: ty.Optional[ty.Any] = None,
+    list_type: ty.Any | None = None,
     coerce_to_default: bool = False,
     deprecated: bool = False,
-    deprecation_reason: ty.Optional[str] = None,
+    deprecation_reason: str | None = None,
 ) -> ty.Any:
     return fields.Computed(
         name,
@@ -196,11 +207,11 @@ class _ComponentManager(collections.abc.MutableMapping):
         return len(self.attributes)
 
     @property
-    def dirty(self):
+    def dirty(self) -> dict[str, ty.Any]:
         """Return a dict of modified attributes"""
         return {key: self.attributes.get(key, None) for key in self._dirty}
 
-    def clean(self, only=None):
+    def clean(self, only: collections.abc.Iterable[str] | None = None) -> None:
         """Signal that the resource no longer has modified attributes.
 
         :param only: an optional set of attributes to no longer consider
@@ -221,18 +232,26 @@ class _Request:
         self.headers = headers
 
 
+class QueryMapping(ty.TypedDict):
+    name: ty_ext.NotRequired[str]
+    type: ty_ext.NotRequired[ty.Callable[[ty.Any, type[ResourceT]], ResourceT]]
+
+
 class QueryParameters:
     def __init__(
         self,
-        *names,
-        include_pagination_defaults=True,
-        **mappings,
+        *names: str,
+        include_pagination_defaults: bool = True,
+        **mappings: str | QueryMapping,
     ):
         """Create a dict of accepted query parameters
 
         :param names: List of strings containing client-side query parameter
             names. Each name in the list maps directly to the name
             expected by the server.
+        :param include_pagination_defaults: If true, include default pagination
+            parameters, ``limit`` and ``marker``. These are the most common
+            query parameters used for listing resources in OpenStack APIs.
         :param mappings: Key-value pairs where the key is the client-side
             name we'll accept here and the value is the name
             the server expects, e.g, ``changes_since=changes-since``.
@@ -241,11 +260,8 @@ class QueryParameters:
             - ``name`` - server-side name,
             - ``type`` - callable to convert from client to server
               representation
-        :param include_pagination_defaults: If true, include default pagination
-            parameters, ``limit`` and ``marker``. These are the most common
-            query parameters used for listing resources in OpenStack APIs.
         """
-        self._mapping: dict[str, ty.Union[str, dict]] = {}
+        self._mapping: dict[str, str | QueryMapping] = {}
         if include_pagination_defaults:
             self._mapping.update({"limit": "limit", "marker": "marker"})
         self._mapping.update({name: name for name in names})
@@ -327,12 +343,28 @@ class QueryParameters:
                 if provide_resource_type:
                     result[name] = type_(value, resource_type)
                 else:
-                    result[name] = type_(value)
+                    result[name] = type_(value)  # type: ignore
             else:
                 result[name] = value
         return result
 
 
+class ResourceMixinProtocol(ty.Protocol):
+    id: str
+    base_path: str
+
+    _body: _ComponentManager
+    _header: _ComponentManager
+    _uri: _ComponentManager
+    _computed: _ComponentManager
+
+    @classmethod
+    def _get_session(cls, session: AdapterT) -> AdapterT: ...
+
+    @classmethod
+    def _get_microversion(cls, session: adapter.Adapter) -> str | None: ...
+
+
 class Resource(dict):
     # TODO(mordred) While this behaves mostly like a munch for the purposes
     # we need, sub-resources, such as Server.security_groups, which is a list
@@ -345,14 +377,14 @@ class Resource(dict):
     # will work properly.
 
     #: Singular form of key for resource.
-    resource_key: ty.Optional[str] = None
+    resource_key: str | None = None
     #: Plural form of key for resource.
-    resources_key: ty.Optional[str] = None
+    resources_key: str | None = None
     #: Key used for pagination links
-    pagination_key: ty.Optional[str] = None
+    pagination_key: str | None = None
 
     #: The ID of this resource.
-    id = Body("id")
+    id: str = Body("id")
 
     #: The name of this resource.
     name: str = Body("name")
@@ -393,16 +425,16 @@ class Resource(dict):
     #: Do calls for this resource require an id
     requires_id = True
     #: Whether create requires an ID (determined from method if None).
-    create_requires_id: ty.Optional[bool] = None
+    create_requires_id: bool | None = None
     #: Whether create should exclude ID in the body of the request.
     create_exclude_id_from_body = False
     #: Do responses for this resource have bodies
     has_body = True
     #: Does create returns a body (if False requires ID), defaults to has_body
-    create_returns_body: ty.Optional[bool] = None
+    create_returns_body: bool | None = None
 
     #: Maximum microversion to use for getting/creating/updating the Resource
-    _max_microversion: ty.Optional[str] = None
+    _max_microversion: str | None = None
     #: API microversion (string or None) this Resource was loaded with
     microversion = None
 
@@ -656,7 +688,7 @@ class Resource(dict):
             res.append((attr, self[attr]))
         return res
 
-    def _update(self, **attrs):
+    def _update(self, **attrs: ty.Any) -> None:
         """Given attributes, update them on this instance
 
         This is intended to be used from within the proxy
@@ -712,7 +744,7 @@ class Resource(dict):
 
         return body, header, uri, computed
 
-    def _update_location(self):
+    def _update_location(self) -> None:
         """Update location to include resource project/zone information.
 
         Location should describe the location of the resource. For some
@@ -737,35 +769,55 @@ class Resource(dict):
         """Compute additional attributes from the remote resource."""
         return {}
 
-    def _consume_body_attrs(self, attrs):
+    def _consume_body_attrs(
+        self, attrs: collections.abc.MutableMapping[str, ty.Any]
+    ) -> dict[str, ty.Any]:
         return self._consume_mapped_attrs(fields.Body, attrs)
 
-    def _consume_header_attrs(self, attrs):
+    def _consume_header_attrs(
+        self, attrs: collections.abc.MutableMapping[str, ty.Any]
+    ) -> dict[str, ty.Any]:
         return self._consume_mapped_attrs(fields.Header, attrs)
 
-    def _consume_uri_attrs(self, attrs):
+    def _consume_uri_attrs(
+        self, attrs: collections.abc.MutableMapping[str, ty.Any]
+    ) -> dict[str, ty.Any]:
         return self._consume_mapped_attrs(fields.URI, attrs)
 
-    def _update_from_body_attrs(self, attrs):
+    def _update_from_body_attrs(
+        self, attrs: collections.abc.MutableMapping[str, ty.Any]
+    ) -> None:
         body = self._consume_body_attrs(attrs)
         self._body.attributes.update(body)
         self._body.clean()
 
-    def _update_from_header_attrs(self, attrs):
+    def _update_from_header_attrs(
+        self, attrs: collections.abc.MutableMapping[str, ty.Any]
+    ) -> None:
         headers = self._consume_header_attrs(attrs)
         self._header.attributes.update(headers)
         self._header.clean()
 
-    def _update_uri_from_attrs(self, attrs):
+    def _update_uri_from_attrs(
+        self, attrs: collections.abc.MutableMapping[str, ty.Any]
+    ) -> None:
         uri = self._consume_uri_attrs(attrs)
         self._uri.attributes.update(uri)
         self._uri.clean()
 
-    def _consume_mapped_attrs(self, mapping_cls, attrs):
+    def _consume_mapped_attrs(
+        self,
+        mapping_cls: type[fields._BaseComponent],
+        attrs: collections.abc.MutableMapping[str, ty.Any],
+    ) -> dict[str, ty.Any]:
         mapping = self._get_mapping(mapping_cls)
         return self._consume_attrs(mapping, attrs)
 
-    def _consume_attrs(self, mapping, attrs):
+    def _consume_attrs(
+        self,
+        mapping: collections.abc.MutableMapping[str, ty.Any],
+        attrs: collections.abc.MutableMapping[str, ty.Any],
+    ) -> dict[str, ty.Any]:
         """Given a mapping and attributes, return relevant matches
 
         This method finds keys in attrs that exist in the mapping, then
@@ -805,7 +857,9 @@ class Resource(dict):
                     self._original_body[attr] = self._body[attr]
 
     @classmethod
-    def _get_mapping(cls, component):
+    def _get_mapping(
+        cls, component: type[fields._BaseComponent]
+    ) -> ty.MutableMapping[str, ty.Any]:
         """Return a dict of attributes of a given component on the class"""
         mapping = component._map_cls()
         ret = component._map_cls()
@@ -856,7 +910,7 @@ class Resource(dict):
         return ""
 
     @staticmethod
-    def _get_id(value):
+    def _get_id(value: Resource | str) -> str:
         """If a value is a Resource, return the canonical ID
 
         This will return either the value specified by `id` or
@@ -870,7 +924,7 @@ class Resource(dict):
             return value
 
     @classmethod
-    def new(cls, **kwargs):
+    def new(cls, **kwargs: ty.Any) -> ty_ext.Self:
         """Create a new instance of this resource.
 
         When creating the instance set the ``_synchronized`` parameter
@@ -901,7 +955,12 @@ class Resource(dict):
         return cls(_synchronized=True, connection=connection, **kwargs)
 
     @classmethod
-    def _from_munch(cls, obj, synchronized=True, connection=None):
+    def _from_munch(
+        cls,
+        obj: dict[str, ty.Union],
+        synchronized: bool = True,
+        connection: connection.Connection | None = None,
+    ) -> ty_ext.Self:
         """Create an instance from a ``utils.Munch`` object.
 
         This is intended as a temporary measure to convert between shade-style
@@ -942,13 +1001,13 @@ class Resource(dict):
 
     def to_dict(
         self,
-        body=True,
-        headers=True,
-        computed=True,
-        ignore_none=False,
-        original_names=False,
-        _to_munch=False,
-    ):
+        body: bool = True,
+        headers: bool = True,
+        computed: bool = True,
+        ignore_none: bool = False,
+        original_names: bool = False,
+        _to_munch: bool = False,
+    ) -> dict[str, ty.Any]:
         """Return a dictionary of this resource's contents
 
         :param bool body: Include the :class:`~openstack.fields.Body`
@@ -968,7 +1027,7 @@ class Resource(dict):
         :return: A dictionary of key/value pairs where keys are named
             as they exist as attributes of this class.
         """
-        mapping: ty.Union[utils.Munch, dict]
+        mapping: utils.Munch | dict
         if _to_munch:
             mapping = utils.Munch()
         else:
@@ -1057,12 +1116,12 @@ class Resource(dict):
 
     def _prepare_request_body(
         self,
-        patch,
-        prepend_key,
+        patch: bool,
+        prepend_key: bool,
         *,
-        resource_request_key=None,
-    ):
-        body: ty.Union[dict[str, ty.Any], list[ty.Any]]
+        resource_request_key: str | None = None,
+    ) -> dict[str, ty.Any] | builtins.list[ty.Any]:
+        body: dict[str, ty.Any] | list[ty.Any]
         if patch:
             if not self._store_unknown_attrs_as_properties:
                 # Default case
@@ -1158,12 +1217,12 @@ class Resource(dict):
 
     def _translate_response(
         self,
-        response,
-        has_body=None,
-        error_message=None,
+        response: requests.Response,
+        has_body: bool | None = None,
+        error_message: str | None = None,
         *,
-        resource_response_key=None,
-    ):
+        resource_response_key: str | None = None,
+    ) -> None:
         """Given a KSA response, inflate this instance with its data
 
         DELETE operations don't return a body, so only try to work
@@ -1218,7 +1277,7 @@ class Resource(dict):
         dict.update(self, self.to_dict())
 
     @classmethod
-    def _get_session(cls, session):
+    def _get_session(cls, session: AdapterT) -> AdapterT:
         """Attempt to get an Adapter from a raw session.
 
         Some older code used conn.session has the session argument to Resource
@@ -1239,7 +1298,7 @@ class Resource(dict):
         )
 
     @classmethod
-    def _get_microversion(cls, session, *, action):
+    def _get_microversion(cls, session: adapter.Adapter) -> str | None:
         """Get microversion to use for the given action.
 
         The base version uses the following logic:
@@ -1254,20 +1313,8 @@ class Resource(dict):
 
         :param session: The session to use for making the request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
-        :param action: One of "fetch", "commit", "create", "delete", "patch".
-        :type action: str
         :return: Microversion as string or ``None``
         """
-        if action not in {
-            'list',
-            'fetch',
-            'commit',
-            'create',
-            'delete',
-            'patch',
-        }:
-            raise ValueError(f'Invalid action: {action}')
-
         if session.default_microversion:
             return session.default_microversion
 
@@ -1275,18 +1322,18 @@ class Resource(dict):
             session, cls._max_microversion
         )
 
+    @classmethod
     def _assert_microversion_for(
-        self,
-        session,
-        action,
-        expected,
-        error_message=None,
-        maximum=None,
-    ):
+        cls,
+        session: adapter.Adapter,
+        expected: str | None,
+        *,
+        error_message: str | None = None,
+        maximum: str | None = None,
+    ) -> str:
         """Enforce that the microversion for action satisfies the requirement.
 
         :param session: :class`keystoneauth1.adapter.Adapter`
-        :param action: One of "fetch", "commit", "create", "delete".
         :param expected: Expected microversion.
         :param error_message: Optional error message with details. Will be
             prepended to the message generated here.
@@ -1296,21 +1343,22 @@ class Resource(dict):
             used for the action is lower than the expected one.
         """
 
-        def _raise(message):
+        def _raise(message: str) -> ty.NoReturn:
             if error_message:
                 error_message.rstrip('.')
                 message = f'{error_message}. {message}'
 
             raise exceptions.NotSupported(message)
 
-        actual = self._get_microversion(session, action=action)
+        actual = cls._get_microversion(session)
 
         if actual is None:
             message = (
                 f"API version {expected} is required, but the default "
-                "version will be used."
+                f"version will be used."
             )
             _raise(message)
+
         actual_n = discover.normalize_version_number(actual)
 
         if expected is not None:
@@ -1318,14 +1366,15 @@ class Resource(dict):
             if actual_n < expected_n:
                 message = (
                     f"API version {expected} is required, but {actual} "
-                    "will be used."
+                    f"will be used."
                 )
                 _raise(message)
+
         if maximum is not None:
             maximum_n = discover.normalize_version_number(maximum)
             # Assume that if a service supports higher versions, it also
             # supports lower ones. Breaks for services that remove old API
-            # versions (which is not something they should do).
+            # versions (which is not something that has been done yet).
             if actual_n > maximum_n:
                 return maximum
 
@@ -1333,15 +1382,15 @@ class Resource(dict):
 
     def create(
         self,
-        session,
-        prepend_key=True,
-        base_path=None,
+        session: adapter.Adapter,
+        prepend_key: bool = True,
+        base_path: str | None = None,
         *,
-        resource_request_key=None,
-        resource_response_key=None,
-        microversion=None,
-        **params,
-    ):
+        resource_request_key: str | None = None,
+        resource_response_key: str | None = None,
+        microversion: str | None = None,
+        **params: ty.Any,
+    ) -> ty_ext.Self:
         """Create a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1368,7 +1417,7 @@ class Resource(dict):
 
         session = self._get_session(session)
         if microversion is None:
-            microversion = self._get_microversion(session, action='create')
+            microversion = self._get_microversion(session)
         requires_id = (
             self.create_requires_id
             if self.create_requires_id is not None
@@ -1425,23 +1474,23 @@ class Resource(dict):
         # direct comparision to False since we need to rule out None
         if self.has_body and self.create_returns_body is False:
             # fetch the body if it's required but not returned by create
-            fetch_kwargs = {}
-            if resource_response_key is not None:
-                fetch_kwargs = {'resource_response_key': resource_response_key}
-            return self.fetch(session, **fetch_kwargs)
+            return self.fetch(
+                session,
+                resource_response_key=resource_response_key,
+            )
         return self
 
     @classmethod
     def bulk_create(
         cls,
-        session,
-        data,
-        prepend_key=True,
-        base_path=None,
+        session: adapter.Adapter,
+        data: builtins.list[dict[str, ty.Any]],
+        prepend_key: bool = True,
+        base_path: str | None = None,
         *,
-        microversion=None,
-        **params,
-    ):
+        microversion: str | None = None,
+        **params: ty.Any,
+    ) -> ty.Generator[ty_ext.Self, None, None]:
         """Create multiple remote resources based on this class and data.
 
         :param session: The session to use for making this request.
@@ -1471,7 +1520,7 @@ class Resource(dict):
 
         session = cls._get_session(session)
         if microversion is None:
-            microversion = cls._get_microversion(session, action='create')
+            microversion = cls._get_microversion(session)
         requires_id = (
             cls.create_requires_id
             if cls.create_requires_id is not None
@@ -1494,17 +1543,24 @@ class Resource(dict):
             # Those objects will be used in case where request doesn't return
             # JSON data representing created resource, and yet it's required
             # to return newly created resource objects.
-            resource = cls.new(connection=session._get_connection(), **attrs)
+            # TODO(stephenfin): Our types say we accept a ksa Adapter, but this
+            # requires an SDK Proxy. Do we update the types or rework this to
+            # support use of an adapter.
+            resource = cls.new(connection=session._get_connection(), **attrs)  # type: ignore
             resources.append(resource)
             request = resource._prepare_request(
                 requires_id=requires_id, base_path=base_path
             )
             _body.append(request.body)
 
-        body: ty.Union[dict[str, ty.Any], list[ty.Any]] = _body
+        body: dict[str, ty.Any] | list[ty.Any] = _body
 
         if prepend_key:
-            assert cls.resources_key
+            if not cls.resources_key:
+                raise exceptions.ResourceFailure(
+                    "Cannot request prepend_key with Unset resources key"
+                )
+
             body = {cls.resources_key: body}
 
         response = method(
@@ -1515,13 +1571,17 @@ class Resource(dict):
             params=params,
         )
         exceptions.raise_from_response(response)
-        data = response.json()
+        json = response.json()
 
         if cls.resources_key:
-            data = data[cls.resources_key]
+            json = json[cls.resources_key]
+        else:
+            json = json
 
-        if not isinstance(data, list):
-            data = [data]
+        if isinstance(data, list):
+            json = json
+        else:
+            json = [json]
 
         has_body = (
             cls.has_body
@@ -1532,26 +1592,29 @@ class Resource(dict):
             return (r.fetch(session) for r in resources)
         else:
             return (
+                # TODO(stephenfin): Our types say we accept a ksa Adapter, but
+                # this requires an SDK Proxy. Do we update the types or rework
+                # this to support use of an adapter.
                 cls.existing(
                     microversion=microversion,
-                    connection=session._get_connection(),
+                    connection=session._get_connection(),  # type: ignore
                     **res_dict,
                 )
-                for res_dict in data
+                for res_dict in json
             )
 
     def fetch(
         self,
-        session,
-        requires_id=True,
-        base_path=None,
-        error_message=None,
-        skip_cache=False,
+        session: adapter.Adapter,
+        requires_id: bool = True,
+        base_path: str | None = None,
+        error_message: str | None = None,
+        skip_cache: bool = False,
         *,
-        resource_response_key=None,
-        microversion=None,
-        **params,
-    ):
+        resource_response_key: str | None = None,
+        microversion: str | None = None,
+        **params: ty.Any,
+    ) -> ty_ext.Self:
         """Get a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1584,7 +1647,7 @@ class Resource(dict):
 
         session = self._get_session(session)
         if microversion is None:
-            microversion = self._get_microversion(session, action='fetch')
+            microversion = self._get_microversion(session)
         self.microversion = microversion
 
         response = session.get(
@@ -1602,7 +1665,13 @@ class Resource(dict):
 
         return self
 
-    def head(self, session, base_path=None, *, microversion=None):
+    def head(
+        self,
+        session: adapter.Adapter,
+        base_path: str | None = None,
+        *,
+        microversion: str | None = None,
+    ) -> ty_ext.Self:
         """Get headers from a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1622,7 +1691,7 @@ class Resource(dict):
 
         session = self._get_session(session)
         if microversion is None:
-            microversion = self._get_microversion(session, action='fetch')
+            microversion = self._get_microversion(session)
         self.microversion = microversion
 
         request = self._prepare_request(base_path=base_path)
@@ -1641,15 +1710,15 @@ class Resource(dict):
 
     def commit(
         self,
-        session,
-        prepend_key=True,
-        has_body=True,
-        retry_on_conflict=None,
-        base_path=None,
+        session: adapter.Adapter,
+        prepend_key: bool = True,
+        has_body: bool = True,
+        retry_on_conflict: bool | None = None,
+        base_path: str | None = None,
         *,
-        microversion=None,
-        **kwargs,
-    ):
+        microversion: str | None = None,
+        **kwargs: ty.Any,
+    ) -> ty_ext.Self:
         """Commit the state of the instance to the remote resource.
 
         :param session: The session to use for making this request.
@@ -1690,7 +1759,7 @@ class Resource(dict):
             **kwargs,
         )
         if microversion is None:
-            microversion = self._get_microversion(session, action='commit')
+            microversion = self._get_microversion(session)
 
         return self._commit(
             session,
@@ -1820,7 +1889,7 @@ class Resource(dict):
             patch=True,
         )
         if microversion is None:
-            microversion = self._get_microversion(session, action='patch')
+            microversion = self._get_microversion(session)
         if patch:
             request.body += self._convert_patch(patch)
 
@@ -1834,8 +1903,13 @@ class Resource(dict):
         )
 
     def delete(
-        self, session, error_message=None, *, microversion=None, **kwargs
-    ):
+        self,
+        session: adapter.Adapter,
+        error_message: str | None = None,
+        *,
+        microversion: str | None = None,
+        **kwargs: ty.Any,
+    ) -> ty_ext.Self:
         """Delete the remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1870,7 +1944,7 @@ class Resource(dict):
         request = self._prepare_request(**kwargs)
         session = self._get_session(session)
         if microversion is None:
-            microversion = self._get_microversion(session, action='delete')
+            microversion = self._get_microversion(session)
 
         return session.delete(
             request.url,
@@ -1881,15 +1955,16 @@ class Resource(dict):
     @classmethod
     def list(
         cls,
-        session,
-        paginated=True,
-        base_path=None,
-        allow_unknown_params=False,
+        session: adapter.Adapter,
+        paginated: bool = True,
+        base_path: str | None = None,
+        allow_unknown_params: bool = False,
         *,
-        microversion=None,
-        headers=None,
-        **params,
-    ):
+        microversion: str | None = None,
+        headers: dict[str, str] | None = None,
+        max_items: int | None = None,
+        **params: ty.Any,
+    ) -> ty.Generator[ty_ext.Self, None, None]:
         """This method is a generator which yields resource objects.
 
         This resource object list generator handles pagination and takes query
@@ -1910,6 +1985,8 @@ class Resource(dict):
         :param str microversion: API version to override the negotiated one.
         :param dict headers: Additional headers to inject into the HTTP
             request.
+        :param int max_items: The maximum number of items to return. Typically
+            this must be used with ``paginated=True``.
         :param dict params: These keyword arguments are passed through the
             :meth:`~openstack.resource.QueryParamter._transpose` method
             to find if any of them match expected query parameters to be sent
@@ -1934,7 +2011,7 @@ class Resource(dict):
         session = cls._get_session(session)
 
         if microversion is None:
-            microversion = cls._get_microversion(session, action='list')
+            microversion = cls._get_microversion(session)
 
         if base_path is None:
             base_path = cls.base_path
@@ -1960,6 +2037,17 @@ class Resource(dict):
         uri = base_path % params
         uri_params = {}
 
+        if max_items and not query_params.get('limit'):
+            # If a user requested max_items but not a limit, set limit to
+            # max_items on the assumption that if (a) the value is smaller than
+            # the maximum server allowed value for limit then we'll be able to
+            # do a single call to get everything, while (b) if the value is
+            # larger then the server will ignore the value (or rather use its
+            # own hardcoded limit) making this is a no-op.
+            # If a user requested both max_items and limit then we assume they
+            # know what they're doing.
+            query_params['limit'] = max_items
+
         limit = query_params.get('limit')
 
         for k, v in params.items():
@@ -2011,6 +2099,11 @@ class Resource(dict):
 
             marker = None
             for raw_resource in resources:
+                # We return as soon as we hit our limit, even if we have items
+                # remaining
+                if max_items and total_yielded >= max_items:
+                    return
+
                 # Do not allow keys called "self" through. Glance chose
                 # to name a key "self", so we need to pop it out because
                 # we can't send it through cls.existing and into the
@@ -2020,9 +2113,12 @@ class Resource(dict):
                 # We want that URI props are available on the resource
                 raw_resource.update(uri_params)
 
+                # TODO(stephenfin): Our types say we accept a ksa Adapter, but
+                # this requires an SDK Proxy. Do we update the types or rework
+                # this to support use of an adapter.
                 value = cls.existing(
                     microversion=microversion,
-                    connection=session._get_connection(),
+                    connection=session._get_connection(),  # type: ignore
                     **raw_resource,
                 )
                 marker = value.id
@@ -2065,7 +2161,7 @@ class Resource(dict):
     @classmethod
     def _get_next_link(cls, uri, response, data, marker, limit, total_yielded):
         next_link = None
-        params = {}
+        params: dict[str, str | list[str] | int] = {}
 
         if isinstance(data, dict):
             pagination_key = cls.pagination_key
@@ -2157,12 +2253,12 @@ class Resource(dict):
         session: adapter.Adapter,
         name_or_id: str,
         ignore_missing: ty.Literal[True] = True,
-        list_base_path: ty.Optional[str] = None,
+        list_base_path: str | None = None,
         *,
-        microversion: ty.Optional[str] = None,
-        all_projects: ty.Optional[bool] = None,
+        microversion: str | None = None,
+        all_projects: bool | None = None,
         **params: ty.Any,
-    ) -> ty.Optional[ty_ext.Self]: ...
+    ) -> ty_ext.Self | None: ...
 
     @ty.overload
     @classmethod
@@ -2171,10 +2267,10 @@ class Resource(dict):
         session: adapter.Adapter,
         name_or_id: str,
         ignore_missing: ty.Literal[False],
-        list_base_path: ty.Optional[str] = None,
+        list_base_path: str | None = None,
         *,
-        microversion: ty.Optional[str] = None,
-        all_projects: ty.Optional[bool] = None,
+        microversion: str | None = None,
+        all_projects: bool | None = None,
         **params: ty.Any,
     ) -> ty_ext.Self: ...
 
@@ -2187,12 +2283,12 @@ class Resource(dict):
         session: adapter.Adapter,
         name_or_id: str,
         ignore_missing: bool,
-        list_base_path: ty.Optional[str] = None,
+        list_base_path: str | None = None,
         *,
-        microversion: ty.Optional[str] = None,
-        all_projects: ty.Optional[bool] = None,
+        microversion: str | None = None,
+        all_projects: bool | None = None,
         **params: ty.Any,
-    ) -> ty.Optional[ty_ext.Self]: ...
+    ) -> ty_ext.Self | None: ...
 
     @classmethod
     def find(
@@ -2200,12 +2296,12 @@ class Resource(dict):
         session: adapter.Adapter,
         name_or_id: str,
         ignore_missing: bool = True,
-        list_base_path: ty.Optional[str] = None,
+        list_base_path: str | None = None,
         *,
-        microversion: ty.Optional[str] = None,
-        all_projects: ty.Optional[bool] = None,
+        microversion: str | None = None,
+        all_projects: bool | None = None,
         **params: ty.Any,
-    ) -> ty.Optional[ty_ext.Self]:
+    ) -> ty_ext.Self | None:
         """Find a resource by its name or id.
 
         :param session: The session to use for making this request.
@@ -2282,24 +2378,21 @@ class Resource(dict):
         )
 
 
-def _normalize_status(status: ty.Optional[str]) -> ty.Optional[str]:
+def _normalize_status(status: str | None) -> str | None:
     if status is not None:
         status = status.lower()
     return status
 
 
-ResourceT = ty.TypeVar('ResourceT', bound=Resource)
-
-
 def wait_for_status(
     session: adapter.Adapter,
     resource: ResourceT,
     status: str,
-    failures: ty.Optional[list[str]] = None,
-    interval: ty.Union[int, float, None] = 2,
-    wait: ty.Optional[int] = None,
+    failures: list[str] | None = None,
+    interval: int | float | None = 2,
+    wait: int | None = None,
     attribute: str = 'status',
-    callback: ty.Optional[ty.Callable[[int], None]] = None,
+    callback: ty.Callable[[int], None] | None = None,
 ) -> ResourceT:
     """Wait for the resource to be in a particular status.
 
@@ -2373,9 +2466,9 @@ def wait_for_status(
 def wait_for_delete(
     session: adapter.Adapter,
     resource: ResourceT,
-    interval: ty.Union[int, float, None] = 2,
-    wait: ty.Optional[int] = None,
-    callback: ty.Optional[ty.Callable[[int], None]] = None,
+    interval: int | float | None = 2,
+    wait: int | None = None,
+    callback: ty.Callable[[int], None] | None = None,
 ) -> ResourceT:
     """Wait for a resource to be deleted.
 
@@ -2394,7 +2487,10 @@ def wait_for_delete(
     orig_resource = resource
     for count in utils.iterate_timeout(
         timeout=wait,
-        message=f"Timeout waiting for {resource.__class__.__name__}:{resource.id} to delete",
+        message=(
+            f"Timeout waiting for {resource.__class__.__name__}:{resource.id} "
+            f"to delete"
+        ),
         wait=interval,
     ):
         try:
