@@ -33,7 +33,7 @@ class TestPort(base.TestCase):
             'allowed_address_pairs': [],
             'admin_state_up': True,
             'network_id': 'test-net-id',
-            'tenant_id': 'test-tenant-id',
+            'project_id': 'test-project-id',
             'binding:vif_details': {},
             'binding:vnic_type': 'normal',
             'binding:vif_type': 'unbound',
@@ -58,7 +58,7 @@ class TestPort(base.TestCase):
             'allowed_address_pairs': [],
             'admin_state_up': True,
             'network_id': 'test-net-id',
-            'tenant_id': 'test-tenant-id',
+            'project_id': 'test-project-id',
             'binding:vif_details': {},
             'extra_dhcp_opts': [],
             'binding:vnic_type': 'normal',
@@ -84,7 +84,7 @@ class TestPort(base.TestCase):
                 'allowed_address_pairs': [],
                 'admin_state_up': True,
                 'network_id': '70c1db1f-b701-45bd-96e0-a313ee3430b3',
-                'tenant_id': '',
+                'project_id': '',
                 'extra_dhcp_opts': [],
                 'binding:vif_details': {
                     'port_filter': True,
@@ -112,7 +112,7 @@ class TestPort(base.TestCase):
                 'allowed_address_pairs': [],
                 'admin_state_up': True,
                 'network_id': 'f27aa545-cbdd-4907-b0c6-c9e8b039dcc2',
-                'tenant_id': 'd397de8a63f341818f198abb0966f6f3',
+                'project_id': 'd397de8a63f341818f198abb0966f6f3',
                 'extra_dhcp_opts': [],
                 'binding:vif_details': {
                     'port_filter': True,
