@@ -21,6 +21,7 @@ from openstack.block_storage.v3 import volume
 from openstack.compute.v2 import _proxy
 from openstack.compute.v2 import aggregate
 from openstack.compute.v2 import availability_zone as az
+from openstack.compute.v2 import console_auth_token
 from openstack.compute.v2 import extension
 from openstack.compute.v2 import flavor
 from openstack.compute.v2 import hypervisor
@@ -43,6 +44,7 @@ from openstack.identity.v3 import project
 from openstack import proxy as proxy_base
 from openstack.tests.unit import base
 from openstack.tests.unit import test_proxy_base
+from openstack import types
 from openstack import warnings as os_warnings
 
 
@@ -644,6 +646,7 @@ class TestVolumeAttachment(TestComputeProxy):
             volume_attachment.VolumeAttachment,
             method_args=[],
             method_kwargs={'server': 'server_id', 'volume': 'volume_id'},
+            expected_args=[None],
             expected_kwargs={
                 'id': 'volume_id',
                 'server_id': 'server_id',
@@ -1370,6 +1373,7 @@ class TestCompute(TestComputeProxy):
             expected_args=[self.proxy],
             expected_kwargs={
                 "host": None,
+                "availability_zone": types.UNSET,
             },
         )
 
@@ -1378,11 +1382,9 @@ class TestCompute(TestComputeProxy):
             "openstack.compute.v2.server.Server.unshelve",
             self.proxy.unshelve_server,
             method_args=["value"],
-            method_kwargs={"host": "HOST2"},
+            method_kwargs={"host": "HOST2", "availability_zone": "AZ2"},
             expected_args=[self.proxy],
-            expected_kwargs={
-                "host": "HOST2",
-            },
+            expected_kwargs={"host": "HOST2", "availability_zone": "AZ2"},
         )
 
     def test_server_trigger_dump(self):
@@ -1567,7 +1569,7 @@ class TestCompute(TestComputeProxy):
 
     def test_force_complete_server_migration(self):
         self._verify(
-            'openstack.compute.v2.server_migration.ServerMigration.force_complete',  # noqa: E501
+            'openstack.compute.v2.server_migration.ServerMigration.force_complete',
             self.proxy.force_complete_server_migration,
             method_args=['server_migration', 'server'],
             expected_args=[self.proxy],
@@ -1636,7 +1638,7 @@ class TestCompute(TestComputeProxy):
         self.verify_list(self.proxy.usages, usage.Usage)
 
     def test_usages__with_kwargs(self):
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc).replace(tzinfo=None)
         start = now - datetime.timedelta(weeks=4)
         end = end = now + datetime.timedelta(days=1)
         self.verify_list(
@@ -1660,7 +1662,7 @@ class TestCompute(TestComputeProxy):
         )
 
     def test_get_usage__with_kwargs(self):
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc).replace(tzinfo=None)
         start = now - datetime.timedelta(weeks=4)
         end = end = now + datetime.timedelta(days=1)
         self._verify(
@@ -1884,3 +1886,11 @@ class TestServerAction(TestComputeProxy):
             method_kwargs={'server': 'server_a'},
             expected_kwargs={'server_id': 'server_a'},
         )
+
+
+class TestValidateConsoleAuthToken(TestComputeProxy):
+    def test_validate_console_auth_token(self):
+        self.verify_get(
+            self.proxy.validate_console_auth_token,
+            console_auth_token.ConsoleAuthToken,
+        )
